package Parse;

my $DEBUG=0;

use strict;
use XML::Parser;
use Tag;
use Render;
use Unicode::String;
use Utf2kp;
use kItrans2utf;
use TamilUtf2Ta;
use TamilItrans2utf;
use hItrans2utf;
use TeluguUtf2Ab;
use TeluguItrans2utf;

#
# Handlers for XML Parser;  This will create a data structure while parsing
# and create a tree
#
# Structure of the tree: each tag gets translated into
# [tagobj, child1, ... childn]
#
# #text# is a pseudo tag that indicates plain text in the file
#

#
# new creates a parse tree
#
#

sub new {
	my $proto=shift;
        my $state=shift;
	my $class=ref($proto)||$proto;
	my $p=new XML::Parser(ErrorContext => 1, ProtocolEncoding => 'UTF-8');
        my $root=[];
	$p->setHandlers(
			Init      => sub{_init_handler(@_, $root, $state)},
			Start     => \&_iml_handler,
			Char      => sub {},
			End       => \&_end_handler,
			CdataStart => sub {},
			CdataEnd  => sub {},
			Final     => \&_final_handler);
        my $inc_parse=$p->parse_start(ProtocolEncoding => 'UTF-8');
        my $self=[$root,$inc_parse];        
	bless $self, $class;
	return $self
}


sub parse{
	my $self=shift->[1];
	my $data=shift;
        if($data eq ""){return}
        print "\t\t<------- pppppaaaaaarrrrr\n" if $DEBUG;
        $self->parse_more($data);
	#eval {$self->parse_more($data)};
        #if($@){$self->set_parse_error; print "ERROR: $@"}
        print "\t\t<------- rrrrrssseeeeeeee\n" if $DEBUG;
}

sub done{
	my $self=shift->[1];
        eval{$self->parse_done()};
}

# Private:  These are called by the parser 
#
#
# The invariant: In the tree, curr_node is the node to whom we will 
# attach the next tag as a child. The stack contains the path from the
# root to the parent of the curr_node. 
# Intialization: In  _iml_handler, curr_node points at the root of the
# tree and the stack is empty.
#

sub _init_handler {
        my $expat = shift;
	$expat->{Tree} = shift;# remember the root
        $expat->{State}= shift; # remember the State
}

sub _iml_handler{
	my ($expat, $tag, %a) = @_;
        my $newlist=$expat->{Tree};
	my $imltag=Tag->new($tag,\%a);
	push @$newlist,$imltag;
	bless $newlist;
        # establish path invariance 
	$expat->{stack} = [];        
        $expat->{curr_node} = $newlist;
        #change handler
        $expat->setHandlers(Start => \&_start_handler);
        Render::begin_tag($expat->{State},$expat->{curr_node});        
}

sub _final_handler{
	my $expat = shift;        
	delete $expat->{stack};
	delete $expat->{curr_node};
	$expat->{Tree}        
}

sub _start_handler{
	my ($expat, $tag, %a) = @_;
	my $tagobj=Tag->new($tag,\%a);
	my $newlist = [$tagobj];
	bless $newlist;        
	push @{$expat->{stack}}, $expat->{curr_node};
	push @{$expat->{curr_node}}, $newlist;
	$expat->{curr_node} = $newlist;
        if($tag=~/^page/){# Within page..  set Char handler 
                $expat->setHandlers(Char => \&_char_handler);
        }
        if($tag=~/^title/){# Within title.. set Char handler 
                $expat->setHandlers(Char => \&_title_handler);
        }
        if($tag=~/^table/){# Within table.. ignore Char handler
                $expat->setHandlers(Char => sub {});        
        }        
        if($tag=~/^td/){# Within td ..  set Char handler 
                $expat->setHandlers(Char => \&_char_handler);
        }                
        Render::begin_tag($expat->{State},$expat->{curr_node});
}

sub _end_handler{
	my ($expat, $tag) = @_;
        Render::end_tag($expat->{State},$expat->{curr_node});
	$expat->{curr_node} = pop  @{$expat->{stack}};
        if($tag=~/^page/){# Outside page..  ignore Char
                $expat->setHandlers(Char => sub {});
        }
        if($tag=~/^title/){# Outside title..  ignore Char handler
                $expat->setHandlers(Char => sub {});
        }
        if($tag=~/^td/){# Outside td.. now ignore Char handler
                $expat->setHandlers(Char => sub {});
        }
        if($tag=~/^table/){# Outside table .. set Char handler 
                $expat->setHandlers(Char => \&_char_handler);        
        }
}


#
# Unlike tags, the character handler is called only once. So we need
# to call both begin_tag and end_tag on this to avoid messing the
# renderer's stack
#

sub _char_handler {
	my ($expat, $tag) = @_;

    print "\t\tb4->|$tag|\n" if $DEBUG;

#    if ($expat->{State}->{langlevel}>0)
#    {
#        $tag = Utf2kp::utf2kp($tag)
#    }

    my $current_lang = $expat->{State}->langstack_top();
    my $current_langmode = $expat->{State}->langmodestack_top();
    if ($current_lang eq "kannada")
    {
        if($current_langmode eq "itrans") {
            $tag=kItrans2utf::itrans2utf($tag);
        }
        $tag = Utf2kp::utf2kp($tag)
    }
    elsif ($current_lang eq "tamil")
    {
        if($current_langmode eq "itrans") {
            $tag=TamilItrans2utf::itrans2utf($tag);
        }
        $tag = TamilUtf2Ta::utf2ta($tag)
    }
    elsif ($current_lang eq "telugu")
    {
        if($current_langmode eq "itrans") {
            $tag=TeluguItrans2utf::itrans2utf($tag);
        }
        $tag = TeluguUtf2Ab::utf2ab($tag)
    }
    elsif ($current_lang eq "hindi")
    {
        if($current_langmode eq "itrans") {
            $tag=hItrans2utf::itrans2utf($tag);
        }
        # FIXME: Add Utf2nd here
        $tag = Unicode::String::utf8($tag)->latin1;
    }
    else
    {
        $tag = Unicode::String::utf8($tag)->latin1;
    }

    print "\t\tafter->|$tag|\n" if $DEBUG;
	my $charobj=Tag->new("#txt#",{text=>$tag});
	my $newlist = [$charobj];
	bless $newlist;
	push @{$expat->{curr_node}},$newlist;
        Render::begin_tag($expat->{State},$newlist);
        Render::end_tag($expat->{State},$newlist);        
}

sub _title_handler {
        my ($expat, $tag) = @_;
        chomp($tag);
        print "\t\t|$tag|\n" if $DEBUG;
	my $charobj=Tag->new("#title#",{text=>$tag});
	my $newlist = [$charobj];        
        bless $newlist;
	push @{$expat->{curr_node}},$newlist;
        Render::begin_tag($expat->{State},$newlist);
        Render::end_tag($expat->{State},$newlist);        
}

#
#  Procedures for traversing the  tree
#
#

sub tag{
	my $node = shift;
	return ($node->[0]->tag);
}

sub attr{
	my $node = shift;
	return ($node->[0]->attr);
}

sub children{
	my $node = shift;
	my @children = @$node;
	shift @children;
	return \@children;
}

sub traverse{
        my $self=shift;
        my ($node, $inc_parse) = @$self;
        _traverse($node);                
}        


sub _traverse{
	my $node = shift;
	my $tag = $node->tag;
	my $attr= $node->attr;
	print "(", $tag," $node\n";
	for my $key (keys %$attr){
		print "* $key =", $attr->{$key},"\n";
	}
	my $children_ref = $node->children;
	for my $dear_child (@$children_ref){_traverse($dear_child)}
	print ")\n";
}

# debug 
sub debug {$DEBUG=1}

#


#sub DESTROY {
#        print "destroying ", shift, "\n";
#        
#}

1;
