package TamilItrans2utf;
use Unicode::String qw(utf8 latin1 ucs2 uchr);

# Basic vowels and consonants
%itranstab = (
    "a"     => 0x0b85,
    "A"     => 0x0b86,
    "i"     => 0x0b87,
    "I"     => 0x0b88,
    "u"     => 0x0b89,
    "U"     => 0x0b8a,
    "e"     => 0x0b8e,
    "E"     => 0x0b8f,
    "ai"    => 0x0b90,
    "o"     => 0x0b92,
    "O"     => 0x0b93,
    "au"    => 0x0b94,
    "q"     => 0x0b83,
    "k"    => 0x0b95,
    "kh"   => 0x0b95,
    "g"    => 0x0b95,
    "gh"   => 0x0b95,
    "~N"   => 0x0b99,
    "ch"   => 0x0b9a,
    "Ch"   => 0x0b9b,
    "j"    => 0x0b9c,
    "jh"   => 0x0b9d,
    "~n"   => 0x0b9e,
    "T"    => 0x0b9f,
    "Th"   => 0x0b9f,
    "D"    => 0x0b9f,
    "Dh"   => 0x0b9f,
    "N"    => 0x0ba3,
    "t"    => 0x0ba4,
    "th"   => 0x0ba4,
    "d"    => 0x0ba4,
    "dh"   => 0x0ba4,
    "n"    => 0x0ba8,
    "p"    => 0x0baa,
    "ph"   => 0x0baa,
    "b"    => 0x0baa,
    "bh"   => 0x0baa,
    "m"    => 0x0bae,
    "y"    => 0x0baf,
    "r"    => 0x0bb0,
    "R"    => 0x0bb1,
    "l"    => 0x0bb2,
    "L"    => 0x0bb3,
    "J"    => 0x0bb4,
    "z"    => 0x0bb4,
    "v"    => 0x0bb5,
    "w"    => 0x0bb5,
    "^n"   => 0x0ba9,
    "sh"   => 0x0bb7,
    "Sh"   => 0x0bb7,
    "s"    => 0x0bb8,
    "h"    => 0x0bb9,
    "x"    => 0x0bba,
    "SRI"  => 0x0bbb,
    "{}"   => 0x0bcd,
);

# Dependent vowels
%itransdeptab = (
    "A"     => 0x0bbe,
    "i"     => 0x0bbf,
    "I"     => 0x0bc0,
    "u"     => 0x0bc1,
    "U"     => 0x0bc2,
    "e"     => 0x0bc6,
    "E"     => 0x0bc7,
    "ai"    => 0x0bc8,
    "o"     => 0x0bca,
    "O"     => 0x0bcb,
    "au"    => 0x0bcc,
    "{}"     => 0x0bcd,
);

sub itrans2utf {
    my $itrans_str = shift;
    my $token_length = 1;
    my $token_begin = 0;
    my $utf_str = Unicode::String->new("");
    my $state = "independent";
    my $encountered = "";

LOOP:
    while ($token_begin < length ($itrans_str)) 
    {
        if($token_length > 3) { return " "; }

        if (substr($itrans_str,$token_begin,1) eq " ")
        {    
            $utf_str->append(uchr(0x20));
            $token_begin++;
        }

        if (exists $itranstab{substr($itrans_str,$token_begin,$token_length)})
        {
            my $char = substr($itrans_str,$token_begin,$token_length);

            if ($char=~/^[aAiIuUeEoOq\{]/) { $encountered = "vowel"; }
            else { $encountered = "consonant"; }

            ($char=~/^[\~kgchnTtpbmyrlvwJzsRjDSx\^]/) && ($token_length == 1) && do
            {
                if(substr($itrans_str,$token_begin+1,1) eq "h")
                {
                    $token_length++;
                }
            };

            if ($char eq "a")
            {
                if($token_begin+1 < length($itrans_str))
                {
                    if((substr($itrans_str,$token_begin+1,1) eq "i") or
                       (substr($itrans_str,$token_begin+1,1) eq "u"))
                    {
                        $token_length++;
                    }
                    elsif($state eq "dependent")
                    {
                        $token_length = 1;
                        $token_begin++;
                        $state = "independent";
                        next LOOP;
                    }
                }
                elsif($state eq "dependent")
                {
                    $token_length = 1;
                    $token_begin++;
                    $state = "independent";
                    next LOOP;
                }
            }

            my $uchar;
            
            if(($encountered eq "vowel") && ($state eq "dependent"))
            {
                $uchar=$itransdeptab{substr($itrans_str,$token_begin,$token_length)};
                $state = "independent";
            }

            else
            {
	            if (($encountered eq "consonant") && ($state eq "dependent") )
        	    {
		        $uchar=$itransdeptab{"."};
        	        $utf_str->append(uchr($uchar));
            	     }

                $uchar=$itranstab{substr($itrans_str,$token_begin,$token_length)};
                if($encountered eq "vowel" || $char eq "SRI") { $state = "independent"; }
                else { $state = "dependent"; }
            }
            $utf_str->append(uchr($uchar));
            $token_begin += $token_length;
            $token_length = 1;
        }
        else
        {
            $token_length++;
        }
    }
	    

    	            if (($encountered eq "consonant") && ($state eq "dependent"))
        	    {
		        $uchar=$itransdeptab{"."};
        	        $utf_str->append(uchr($uchar));
            	     }

    return $utf_str->utf8;
}
