package TeluguItrans2utf;
use Unicode::String qw(utf8 latin1 ucs2 uchr);

# Basic vowels and consonants
%itranstab = (
    "a"     => 0x0c05,
    "A"     => 0x0c06,
    "i"     => 0x0c07,
    "I"     => 0x0c08,
    "u"     => 0x0c09,
    "U"     => 0x0c0a,
    "RRu"    => 0x0c0b,
    "RRU"   => 0x0c0c,
    "e"     => 0x0c0e,
    "E"     => 0x0c0f,
    "ai"    => 0x0c10,
    "o"     => 0x0c12,
    "O"     => 0x0c13,
    "au"    => 0x0c14,
    ".n"    => 0x0c02,
    "H"     => 0x0c03,
    "k"    => 0x0c15,
    "kh"   => 0x0c16,
    "g"    => 0x0c17,
    "gh"   => 0x0c18,
    "~N"   => 0x0c19,
    "ch"    => 0x0c1a,
    "Ch"   => 0x0c1b,
    "j"    => 0x0c1c,
    "jh"   => 0x0c1d,
    "~n"   => 0x0c1e,
    "T"   => 0x0c1f,
    "Th"  => 0x0c20,
    "D"   => 0x0c21,
    "Dh"  => 0x0c22,
    "N"   => 0x0c23,
    "t"    => 0x0c24,
    "th"   => 0x0c25,
    "d"    => 0x0c26,
    "dh"   => 0x0c27,
    "n"    => 0x0c28,
    "p"    => 0x0c2a,
    "ph"   => 0x0c2b,
    "b"    => 0x0c2c,
    "bh"   => 0x0c2d,
    "m"    => 0x0c2e,
    "y"    => 0x0c2f,
    "r"    => 0x0c30,
    "rh"   => 0x0c31,
    "l"    => 0x0c32,
    "L"   => 0x0c33,
    "v"    => 0x0c35,
    "sh"   => 0x0c36,
    "Sh"   => 0x0c37,
    "s"    => 0x0c38,
    "h"    => 0x0c39,
    "0"     => 0x0c66,
    "1"     => 0x0c67,
    "2"     => 0x0c68,
    "3"     => 0x0c69,
    "4"     => 0x0c6a,
    "5"     => 0x0c6b,
    "6"     => 0x0c6c,
    "7"     => 0x0c6d,
    "8"     => 0x0c6e,
    "9"     => 0x0c6f,
#    "RRU"   => 0x0c60,
#    "LLU"   => 0x0c61
    "{}"    => 0x0c4d
);



# Dependent vowels
%itransdeptab = (
    "A"     => 0x0c3e,
    "i"     => 0x0c3f,
    "I"     => 0x0c40,
    "u"     => 0x0c41,
    "U"     => 0x0c42,
    "RRu"    => 0x0c43,
    "RRU"    => 0x0c44,
    "e"     => 0x0c46,
    "E"     => 0x0c47,
    "ai"    => 0x0c48,
    "o"     => 0x0c4a,
    "O"     => 0x0c4b,
    "au"    => 0x0c4c,
    "{}"    => 0x0c4d,
    ".n"    => 0x0c02,
    "H"    => 0x0c03
);


sub itrans2utf {
    my $itrans_str = shift;
    my $token_length = 1;
    my $token_begin = 0;
#    my $utf_str = "";
    my $utf_str = Unicode::String->new("");
    my $state = "independent";
    my $encountered = "";

LOOP:
    while ($token_begin < length ($itrans_str)) 
    {
        if($token_length > 3) { return ""; }

        if (substr($itrans_str,$token_begin,1) eq " ")
        {
#            $utf_str.=" ";
            $utf_str->append(uchr(0x20));
            $token_begin++;
	    next LOOP;
        }

        if (exists $itranstab{substr($itrans_str,$token_begin,$token_length)})
        {
            my $char = substr($itrans_str,$token_begin,$token_length);

            if ($char=~/^[aAiIuUReEoOH\.\{]/) { $encountered = "vowel"; }
            else { $encountered = "consonant"; }

            ($char=~/^[kgcjtTdDpbsS]/) && ($token_length == 1) && do
            {
                if(substr($itrans_str,$token_begin+1,1) eq "h")
                {
                    $token_length++;
                }
            };

            if ($char eq "a")
            {
                if($token_begin+1 < length($itrans_str))
                {
                    if((substr($itrans_str,$token_begin+1,1) eq "i") or
                       (substr($itrans_str,$token_begin+1,1) eq "u"))
                    {
                        $token_length++;
                    }
                    elsif($state eq "dependent")
                    {
                        $token_length = 1;
                        $token_begin++;
                        $state = "independent";
                        next LOOP;
                    }
                }
                elsif($state eq "dependent")
                {
                    $token_length = 1;
                    $token_begin++;
                    $state = "independent";
                    next LOOP;
                }
            }

            my $uchar;
            
            if(($encountered eq "vowel") && ($state eq "dependent"))
            {
                $uchar=$itransdeptab{substr($itrans_str,$token_begin,$token_length)};
                $state = "independent";
            }
            else
            {
                $uchar=$itranstab{substr($itrans_str,$token_begin,$token_length)};
                if($encountered eq "vowel") { $state = "independent"; }
                else { $state = "dependent"; }
            }
            my $us = uchr($uchar);
#            $utf_str .= $us->utf8;
            $utf_str->append(uchr($uchar));
            $token_begin += $token_length;
            $token_length = 1;
        }
        else
        {
            $token_length++;
        }
    }

#    return $utf_str;
    return $utf_str->utf8;
}
