#!/usr/bin/perl

package TeluguUtf2Ab;

# This exports a routine called utf2ab, which reads a UTF8 Telugu
# string and returns a string in ISO8859-1 Telugu Prabha font

use Unicode::String qw(utf8 latin1 utf16);
my $DEBUG=0;


@abvowels = (
    # Vowels
    "h",    # 0x0C05, Telugu letter A
    "H",    # 0x0C06, Telugu letter AA
    "x",    # 0x0C07, Telugu letter I
    "X",    # 0x0C08, Telugu letter II
    "w",    # 0x0C09, Telugu letter U
    "W",    # 0x0C0A, Telugu letter UU
    "",    # 0x0C0B, Telugu letter R
    "",    # 0x0C0C, Telugu letter L
    "",     # 0x0C0D, <reserved>
    "~",    # 0x0C0E, Telugu letter E
    "`",    # 0x0C0F, Telugu letter EE
    "@",    # 0x0C10, Telugu letter AI
    "",     # 0x0C11, <reserved>
    "\\",   # 0x0C12, Telugu letter O
    "|",    # 0x0C13, Telugu letter OO
    "\\^",  # 0x0C14, Telugu letter AU
);

@abconsonants = (
    # Consonants
    "k",    # 0x0C15, Telugu letter KA
    "K",    # 0x0C16, Telugu letter KHA
    "g",    # 0x0C17, Telugu letter GA
    "G",    # 0x0C18, Telugu letter GHA
    "{",    # 0x0C19, Telugu letter NGA
    "c",    # 0x0C1A, Telugu letter CA
    "C",    # 0x0C1B, Telugu letter CHA
    "j",    # 0x0C1C, Telugu letter JA
    "J",    # 0x0C1D, Telugu letter JHA
    "}",    # 0x0C1E, Telugu letter NYA
    "t",    # 0x0C1F, Telugu letter TTA
    "T",    # 0x0C20, Telugu letter TTHA
    "d",    # 0x0C21, Telugu letter DDA
    "D",    # 0x0C22, Telugu letter DDHA
    "N",    # 0x0C23, Telugu letter NNA
    "f",    # 0x0C24, Telugu letter TA
    "F",    # 0x0C25, Telugu letter THA
    "z",    # 0x0C26, Telugu letter DA
    "Z",    # 0x0C27, Telugu letter DHA
    "n",    # 0x0C28, Telugu letter NA
    "",     # 0x0C29, <reserved>
    "p",    # 0x0C2A, Telugu letter PA
    "P",    # 0x0C2B, Telugu letter PHA
    "b",    # 0x0C2C, Telugu letter BA
    "B",    # 0x0C2D, Telugu letter BHA
    "m",    # 0x0C2E, Telugu letter MA
    "y",    # 0x0C2F, Telugu letter YA
    "r",    # 0x0C30, Telugu letter RA
    "R",    # 0x0C31, Telugu letter RRA
    "l",    # 0x0C32, Telugu letter LA
    "L",    # 0x0C33, Telugu letter LLA
    "",     # 0x0C34, <reserved>
    "v",    # 0x0C35, Telugu letter VA
    "S",    # 0x0C36, Telugu letter SHA
    "q",    # 0x0C37, Telugu letter SSA
    "s",    # 0x0C38, Telugu letter SA
    "Q",     # 0x0C39, Telugu letter HA
);

@abvottus = (
    # Vottus
    "",    # 0x0C15, Telugu letter KA
    "",    # 0x0C16, Telugu letter KHA
    "",    # 0x0C17, Telugu letter GA
    "",    # 0x0C18, Telugu letter GHA
    "{",   # 0x0C19, Telugu letter NGA
    "",    # 0x0C1A, Telugu letter CA
    "",    # 0x0C1B, Telugu letter CHA
    "",    # 0x0C1C, Telugu letter JA
    "",  # 0x0C1D, Telugu letter JHA
    "}",    # 0x0C1E, Telugu letter NYA
    "",    # 0x0C1F, Telugu letter TTA
    "",    # 0x0C20, Telugu letter TTHA
    "",    # 0x0C21, Telugu letter DDA
    "",    # 0x0C22, Telugu letter DDHA
    "",    # 0x0C23, Telugu letter NNA
    "",    # 0x0C24, Telugu letter TA
    "",    # 0x0C25, Telugu letter THA
    "",    # 0x0C26, Telugu letter DA
    "",    # 0x0C27, Telugu letter DHA
    "",    # 0x0C28, Telugu letter NA
    "",     # 0x0C29, <reserved>
    "",    # 0x0C2A, Telugu letter PA
    "",    # 0x0C2B, Telugu letter PHA
    "",    # 0x0C2C, Telugu letter BA
    "",    # 0x0C2D, Telugu letter BHA
    "",    # 0x0C2E, Telugu letter MA
    "",    # 0x0C2F, Telugu letter YA
    "\$",    # 0x0C30, Telugu letter RA
    "",    # 0x0C31, Telugu letter RRA
    "",    # 0x0C32, Telugu letter LA
    "",    # 0x0C33, Telugu letter LLA
    "",     # 0x0C34, <reserved>
    "",    # 0x0C35, Telugu letter VA
    "",    # 0x0C36, Telugu letter SHA
    "",    # 0x0C37, Telugu letter SSA
    "",    # 0x0C38, Telugu letter SA
    "",    # 0x0C39, Telugu letter HA
);

@abanuvis=(
    # Anusvara and Visarga
    ";",    # 0x0C02, Telugu sign anusvara
    ":",    # 0x0C03, Telugu sign visarga
);

@abdepend=(
    "A",    # 0x0C3E, Telugu vowel sign AA
    "i",    # 0x0C3F, Telugu vowel sign I
    "I",    # 0x0C40, Telugu vowel sign II
    "u",    # 0x0C41, Telugu vowel sign U
    "U",    # 0x0C42, Telugu vowel sign UU
    "",    # 0x0C43, Telugu vowel sign R
    "A",   # 0x0C44, Telugu vowel sign RR
    "",    # 0x0C45, <reserved>
    "e",    # 0x0C46, Telugu vowel sign E
    "E",    # 0x0C47, Telugu vowel sign EE
    "]",    # 0x0C40, Telugu vowel sign AI
    "",    # 0x0C49, <reserved>
    "o",    # 0x0C4A, Telugu vowel sign O
    "O",    # 0x0C43, Telugu vowel sign OO
    "^",    # 0x0C4C, Telugu vowel sign AU
    "'",    # 0x0C4D, Telugu sign Virama
    "",    # 0x0C4E, <reserved>
    "",    # 0x0C4F, <reserved>
    "",    # 0x0C50, <reserved>
    "",    # 0x0C51, <reserved>
    "",    # 0x0C52, <reserved>
    "",    # 0x0C53, <reserved>
    "",    # 0x0C54, <reserved>
    "",    # 0x0C55, Telugu length mark
    "",    # 0x0C56, Telugu AI length mark
);

%abdigits=(
    "0",    # 0x0C66, Telugu digit zero
    "1",    # 0x0C67, Telugu digit one
    "2",    # 0x0C68, Telugu digit two
    "3",    # 0x0C69, Telugu digit three
    "4",    # 0x0C6A, Telugu digit four
    "5",    # 0x0C6B, Telugu digit five
    "6",    # 0x0C6C, Telugu digit six
    "7",    # 0x0C6D, Telugu digit seven
    "8",    # 0x0C6E, Telugu digit eight
    "9",    # 0x0C6F, Telugu digit nine
);

sub is_vowel {
    my $ucs_char = shift;

    if(($ucs_char>=0x0c05 && $ucs_char<=0x0c14) || ($ucs_char==0xc60))
    {
        return 1;
    }
    return 0;
}

sub is_consonant {
    my $ucs_char = shift;

    if($ucs_char>=0x0c15 && $ucs_char<=0x0c39)
    {
        return 1;
    }

    return 0;
}

sub is_digit {
    my $ucs_char = shift;

    if($ucs_char>=0x0c66 && $ucs_char<=0x0c6f)
    {
        return 1;
    }

    return 0;
}

sub is_dependent {
    my $ucs_char = shift;

    if(($ucs_char>=0x0c3e && $ucs_char<=0x0c56)
        || ($ucs_char==0x0c02) || ($ucs_char==0x0c03))
    {
        return 1;
    }

    return 0;
}

sub is_telugu {
    my $ucs_char = shift;

    if(($ucs_char>=0x0c01) && ($ucs_char<=0x0c6f))
    {
        return 1;
    }

    return 0;
}

sub is_ascii {
    my $ucs_char = shift;

    if(($ucs_char>=0x1) && ($ucs_char<=0x07f))
    {
        return 1;
    }

    return 0;
}

sub utf2ab {
    my $utf_str = shift;
    my $i;
    my $ucs_str = Unicode::String->new($utf_str);
    my $ab_str = "";

    print "Length: ".$ucs_str->length."\n" if $DEBUG;
    print "Hex:".$ucs_str->hex."\n" if $DEBUG;
    print "Names:" if $DEBUG;
    print $ucs_str->name if $DEBUG;
    print "\n" if $DEBUG;

LOOP:
    for($i=0;$i<$ucs_str->length;$i++)
    {
        #printf "%x\n", $ucs_str->substr($i,1)->ord;
        if ($ucs_str->substr($i,1)->ord == 0x0)
        {
            return $ab_str;
        }
        
        if (!is_telugu($ucs_str->substr($i,1)->ord))
        {
            $ab_str.=$ucs_str->substr($i,1)->latin1;
            next LOOP;
        }
        else
        {
            my $encountered;
            my $ucs_char=$ucs_str->substr($i,1)->ord;
            my $ucs_next_char=0;
            my $ucs_next_next_char=0;

            if ($i+1 < $ucs_str->length)
            {
                $ucs_next_char=$ucs_str->substr($i+1,1)->ord;
            }

            if ($i+2 < $ucs_str->length)
            {
                $ucs_next_next_char=$ucs_str->substr($i+2,1)->ord;
            }
            
            if(is_vowel($ucs_char))
            {
                $ab_str .= $abvowels[$ucs_char - 0x0c05];
                if($ucs_char == 0x0c05)
                {
                    if($ucs_next_char==0x0c02) { $ab_str.=$abanuvis[0]; $i++; }
                    if($ucs_next_char==0x0c03) { $ab_str.=$abanuvis[1]; $i++; }
                }
                next LOOP;
            }
            elsif(is_consonant($ucs_char))
            {
                if(is_consonant($ucs_next_char) || ($ucs_next_char == 0x20) ||
                    ($ucs_next_char == 0x0) || (is_digit($ucs_next_char)))
                {
                    $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                    if(($ucs_char == 0x0c16) || ($ucs_char == 0x0c1c) ||
                        ($ucs_char == 0x0c23) || ($ucs_char == 0x0c2c) ||
                        ($ucs_char == 0x0c31) || ($ucs_char == 0x0c32))
                    { }
                    else
                    {
                        $ab_str.="a";
                    }
                    next LOOP;
                }
                elsif(is_dependent($ucs_next_char))
                {
                    if(($ucs_next_char == 0x0c41) ||
                        ($ucs_next_char == 0x0c42))
                    {
                        $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                        if(($ucs_char == 0x0c16) || ($ucs_char == 0x0c1c) ||
                            ($ucs_char == 0x0c23) || ($ucs_char == 0x0c2c) ||
                            ($ucs_char == 0x0c31) || ($ucs_char == 0x0c32))
                        { }
                        else
                        {
                            $ab_str.="a";
                        }
                        $ab_str.=$abdepend[$ucs_next_char - 0x0c3e];
                        $i++;
                        next LOOP;
                    }
                    elsif(($ucs_next_char == 0x0c02) ||
                        ($ucs_next_char == 0x0c03))
                    {
                        $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                        $ab_str.="a";
                        $ab_str.=$abanuvis[$ucs_next_char - 0x0c02];
                        $i++;
                        next LOOP;
                    }
                    elsif($ucs_next_char == 0x0c4d)
                    {
                        if( ($ucs_next_next_char == 0x20) ||
                            ($ucs_next_next_char == 0x0))
                        {
                            $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                            $ab_str.=" ";
                            $ab_str.=$abdepend[$ucs_next_char - 0x0c3e];
                            $i++;
                            next LOOP;
                        }
                        elsif(is_consonant($ucs_next_next_char))
                        {
                            my $ucs_n3_char = 0;
                            
                            if($i+3 < $ucs_str->length)
                            {
                                $ucs_n3_char=$ucs_str->substr($i+3,1)->ord;
                            }

                            if(is_consonant($ucs_n3_char) || 
                                ($ucs_n3_char == 0x0c02) ||
                                ($ucs_n3_char == 0x0c03) ||
                                ($ucs_n3_char == 0x20) ||
                                ($ucs_n3_char == 0x0))
                            {
                                $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                                if(($ucs_char == 0x0c16) || ($ucs_char == 0x0c1c) ||
                                    ($ucs_char == 0x0c23) || ($ucs_char == 0x0c2c) ||
                                    ($ucs_char == 0x0c31) || ($ucs_char == 0x0c32))
                                { }
                                else
                                {
                                    $ab_str.="a";
                                }
                                $ab_str.=$abvottus[$ucs_next_next_char - 0x0c15];
                                $i+=2;
                                next LOOP;
                            }
                            elsif(is_dependent($ucs_n3_char))
                            {
                                if(($ucs_n3_char==0x0c41) ||
                                    ($ucs_n3_char==0x0c42))
                                {
                                    $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                                    if(($ucs_char == 0x0c16) ||
                                        ($ucs_char == 0x0c1c) ||
                                        ($ucs_char == 0x0c23) ||
                                        ($ucs_char == 0x0c2c) ||
                                        ($ucs_char == 0x0c31) ||
                                        ($ucs_char == 0x0c32))
                                    {}
                                    else
                                    {
                                        $ab_str.="a";
                                    }
                                    $ab_str.=$abdepend[$ucs_n3_char - 0x0c3e];
                                    $ab_str.=$abvottus[$ucs_next_next_char - 0x0c15];
                                    $i+=3;
                                    next LOOP;
                                }
                                else
                                {
                                    $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                                    $ab_str.=" ";
                                    $ab_str.=$abdepend[$ucs_n3_char - 0x0c3e];
                                    $ab_str.=$abvottus[$ucs_next_next_char - 0x0c15];
                                    $i+=3;
                                    next LOOP;
                                }
                            }
                            else
                            {
                                return "";
                            }
                        }
                        else
                        {
                            return "";
                        }
                    }
                    else
                    {
                        $ab_str.=$abconsonants[$ucs_char - 0x0c15];
                        $ab_str.=" ";
                        $ab_str.=$abdepend[$ucs_next_char - 0x0c3e];
                        $i++;
                        next LOOP;
                    }
                }
                else
                {
                    return "";
                }
            }
            elsif(is_digit($ucs_char))
            {
                $ab_str .= $abdigits[$ucs_char - 0x0c66];
                next LOOP;
            }
            elsif(($ucs_char==0x0c02 ) || ($ucs_char==0x0c03))
            {
                $ab_str.=$abanuvis[$ucs_char - 0x0c02];
                next LOOP;
            }
            else
            {
                return "";
            }
        }
    }
    return $ab_str;
}
