#!/usr/bin/perl

package Utf2kp;

# This exports a routine called utf2kp, which reads a UTF8 Kannada
# string and returns a string in ISO8859-1 Kannada Prabha font

use Unicode::String qw(utf8 latin1 utf16);
my $DEBUG=0;


@kpvowels = (
    # Vowels
    "@",    # 0x0C85, Kannada letter A
    "A",    # 0x0C86, Kannada letter AA
    "B",    # 0x0C87, Kannada letter I
    "C",    # 0x0C88, Kannada letter II
    "D",    # 0x0C89, Kannada letter U
    "E",    # 0x0C8A, Kannada letter UU
    "F",   # 0x0C8B, Kannada letter R
    "",     # 0x0C8C, Kannada letter L # Undefined in Kpnews
    "",     # 0x0C8D, <reserved>
    "G",    # 0x0C8E, Kannada letter E
    "H",    # 0x0C8F, Kannada letter EE
    "I",    # 0x0C90, Kannada letter AI
    "",     # 0x0C91, <reserved>
    "J",    # 0x0C92, Kannada letter O
    "K",    # 0x0C93, Kannada letter OO
    "L",    # 0x0C94, Kannada letter AU
);

@kpconsonants = (
    # Consonants
    "O",    # 0x0C95, Kannada letter KA
    "R",    # 0x0C96, Kannada letter KHA
    "V",    # 0x0C97, Kannada letter GA
    "Y",    # 0x0C98, Kannada letter GHA
    "\\",   # 0x0C99, Kannada letter NGA
    "^",    # 0x0C9A, Kannada letter CA
    "a",    # 0x0C9B, Kannada letter CHA
    "d",    # 0x0C9C, Kannada letter JA
    "h",  # 0x0C9D, Kannada letter JHA
    "j",    # 0x0C9E, Kannada letter NYA
    "l",    # 0x0C9F, Kannada letter TTA
    "p",    # 0x0CA0, Kannada letter TTHA
    "s",    # 0x0CA1, Kannada letter DDA
    "v",    # 0x0CA2, Kannada letter DDHA
    "y",    # 0x0CA3, Kannada letter NNA
    "}",    # 0x0CA4, Kannada letter TA
    "",    # 0x0CA5, Kannada letter THA
    "",    # 0x0CA6, Kannada letter DA
    "",    # 0x0CA7, Kannada letter DHA
    "",    # 0x0CA8, Kannada letter NA
    "",     # 0x0CA9, <reserved>
    "",    # 0x0CAA, Kannada letter PA
    "",    # 0x0CAB, Kannada letter PHA
    "",    # 0x0CAC, Kannada letter BA
    "",    # 0x0CAD, Kannada letter BHA
    "",  # 0x0CAE, Kannada letter MA
    "",  # 0x0CAF, Kannada letter YA
    "",    # 0x0CB0, Kannada letter RA
    "",     # 0x0CB1, Kannada letter RRA # Undefined in Kpnews
    "",    # 0x0CB2, Kannada letter LA
    "",     # 0x0CB3, Kannada letter LLA
    "",     # 0x0CB4, <reserved>
    "",    # 0x0CB5, Kannada letter VA
    "",    # 0x0CB6, Kannada letter SHA
    "",    # 0x0CB7, Kannada letter SSA
    "",    # 0x0CB8, Kannada letter SA
    "",    # 0x0CB9, Kannada letter HA
);

@kpIdeps = (
    # The I dependent consonants
    "P",    # 0x0C95, Kannada letter KA
    "T",    # 0x0C96, Kannada letter KHA
    "W",    # 0x0C97, Kannada letter GA
    "Z",    # 0x0C98, Kannada letter GHA
    "",   # 0x0C99, Kannada letter NGA
    "_",    # 0x0C9A, Kannada letter CA
    "b",    # 0x0C9B, Kannada letter CHA
    "f",    # 0x0C9C, Kannada letter JA
    "h",  # 0x0C9D, Kannada letter JHA
    "",    # 0x0C9E, Kannada letter NYA
    "n",    # 0x0C9F, Kannada letter TTA
    "q",    # 0x0CA0, Kannada letter TTHA
    "t",    # 0x0CA1, Kannada letter DDA
    "w",    # 0x0CA2, Kannada letter DDHA
    "z",    # 0x0CA3, Kannada letter NNA
    "~",    # 0x0CA4, Kannada letter TA
    "",    # 0x0CA5, Kannada letter THA
    "",    # 0x0CA6, Kannada letter DA
    "",    # 0x0CA7, Kannada letter DHA
    "",    # 0x0CA8, Kannada letter NA
    "",     # 0x0CA9, <reserved>
    "",    # 0x0CAA, Kannada letter PA
    "",    # 0x0CAB, Kannada letter PHA
    "",    # 0x0CAC, Kannada letter BA
    "",    # 0x0CAD, Kannada letter BHA
    "",  # 0x0CAE, Kannada letter MA
    "",  # 0x0CAF, Kannada letter YA
    "",    # 0x0CB0, Kannada letter RA
    "",     # 0x0CB1, Kannada letter RRA # Undefined in Kpnews
    "",    # 0x0CB2, Kannada letter LA
    "",     # 0x0CB3, Kannada letter LLA
    "",     # 0x0CB4, <reserved>
    "",    # 0x0CB5, Kannada letter VA
    "",    # 0x0CB6, Kannada letter SHA
    "",    # 0x0CB7, Kannada letter SSA
    "",    # 0x0CB8, Kannada letter SA
    "",    # 0x0CB9, Kannada letter HA
);

@kpvottus = (
    # Vottus
    "Q",    # 0x0C95, Kannada letter KA
    "U",    # 0x0C96, Kannada letter KHA
    "X",    # 0x0C97, Kannada letter GA
    "",    # 0x0C98, Kannada letter GHA
    "",   # 0x0C99, Kannada letter NGA
    "",    # 0x0C9A, Kannada letter CA
    "c",    # 0x0C9B, Kannada letter CHA
    "g",    # 0x0C9C, Kannada letter JA
    "i",  # 0x0C9D, Kannada letter JHA
    "k",    # 0x0C9E, Kannada letter NYA
    "o",    # 0x0C9F, Kannada letter TTA
    "r",    # 0x0CA0, Kannada letter TTHA
    "u",    # 0x0CA1, Kannada letter DDA
    "x",    # 0x0CA2, Kannada letter DDHA
    "|",    # 0x0CA3, Kannada letter NNA
    "",    # 0x0CA4, Kannada letter TA
    "",    # 0x0CA5, Kannada letter THA
    "",    # 0x0CA6, Kannada letter DA
    "",    # 0x0CA7, Kannada letter DHA
    "",    # 0x0CA8, Kannada letter NA
    "",     # 0x0CA9, <reserved>
    "",    # 0x0CAA, Kannada letter PA
    "",    # 0x0CAB, Kannada letter PHA
    "",    # 0x0CAC, Kannada letter BA
    "",    # 0x0CAD, Kannada letter BHA
    "",  # 0x0CAE, Kannada letter MA
    "",  # 0x0CAF, Kannada letter YA
    "",    # 0x0CB0, Kannada letter RA
    "",     # 0x0CB1, Kannada letter RRA # Undefined in Kpnews
    "",    # 0x0CB2, Kannada letter LA
    "",     # 0x0CB3, Kannada letter LLA
    "",     # 0x0CB4, <reserved>
    "",    # 0x0CB5, Kannada letter VA
    "",    # 0x0CB6, Kannada letter SHA
    "",    # 0x0CB7, Kannada letter SSA
    "",    # 0x0CB8, Kannada letter SA
    "",    # 0x0CB9, Kannada letter HA
);

@kpanuvis=(
    # Anusvara and Visarga
    "M",    # 0x0C82, Kannada sign anusvara
    "N",    # 0x0C83, Kannada sign visarga
);

@kpdepend=(
    "",    # 0x0CBE, Kannada vowel sign AA
    "",     # 0x0CBF, Kannada vowel sign I
    "",    # 0x0CC0, Kannada vowel sign II
    "",    # 0x0CC1, Kannada vowel sign U
    "",    # 0x0CC2, Kannada vowel sign UU
    "",    # 0x0CC3, Kannada vowel sign R
    "",    # 0x0CC4, Kannada vowel sign RR
    "",     # 0x0CC5, <reserved>
    "",    # 0x0CC6, Kannada vowel sign E
    "",   # 0x0CC7, Kannada vowel sign EE
    "",   # 0x0CC8, Kannada vowel sign AI
    "",     # 0x0CC9, <reserved>
    "",   # 0x0CCA, Kannada vowel sign O
    "",  # 0x0CCB, Kannada vowel sign OO
    "",    # 0x0CCC, Kannada vowel sign AU
    "",    # 0x0CCD, Kannada sign Virama
    "",     # 0x0CCE, <reserved>
    "",     # 0x0CCF, <reserved>
    "",     # 0x0CD0, <reserved>
    "",     # 0x0CD1, <reserved>
    "",     # 0x0CD2, <reserved>
    "",     # 0x0CD3, <reserved>
    "",     # 0x0CD4, <reserved>
    "",    # 0x0CD5, Kannada length mark
    "",    # 0x0CD6, Kannada AI length mark
);

%kpdigits=(
    "0",    # 0x0CE6, Kannada digit zero
    "1",    # 0x0CE7, Kannada digit one
    "2",    # 0x0CE8, Kannada digit two
    "3",    # 0x0CE9, Kannada digit three
    "4",    # 0x0CEA, Kannada digit four
    "5",    # 0x0CEB, Kannada digit five
    "6",    # 0x0CEC, Kannada digit six
    "7",    # 0x0CED, Kannada digit seven
    "8",    # 0x0CEE, Kannada digit eight
    "9",    # 0x0CEF, Kannada digit nine
);

sub is_vowel {
    my $ucs_char = shift;

    if(($ucs_char>=0x0c85 && $ucs_char<=0x0c94) || ($ucs_char==0xce0))
    {
        return 1;
    }

    return 0;
}

sub is_consonant {
    my $ucs_char = shift;

    if($ucs_char>=0x0c95 && $ucs_char<=0x0cb9)
    {
        return 1;
    }

    return 0;
}

sub is_digit {
    my $ucs_char = shift;

    if($ucs_char>=0x0ce6 && $ucs_char<=0x0cef)
    {
        return 1;
    }

    return 0;
}

sub is_dependent {
    my $ucs_char = shift;

    if(($ucs_char>=0x0cbe && $ucs_char<=0x0cd6)
        || ($ucs_char==0x0c82) || ($ucs_char==0x0c83))
    {
        return 1;
    }

    return 0;
}

sub is_kannada {
    my $ucs_char = shift;

    if(($ucs_char>=0x0c82) && ($ucs_char<=0x0cef))
    {
        return 1;
    }

    return 0;
}

sub is_ascii {
    my $ucs_char = shift;

    if(($ucs_char>=0x1) && ($ucs_char<=0x07f))
    {
        return 1;
    }

    return 0;
}

sub is_kannada {
    my $ucs_char = shift;

    if(($ucs_char>=0x0c80) && ($ucs_char<=0x0cff))
    {
        return 1;
    }

    return 0;
}

sub utf2kp {
    my $utf_str = shift;
    my $i;
    my $ucs_str = Unicode::String->new($utf_str);
    my $kp_str = "";

    print "Length: ".$ucs_str->length."\n" if $DEBUG;
    print "Hex:".$ucs_str->hex."\n" if $DEBUG;
    print "Names:".$ucs_str->name."\n" if $DEBUG;

LOOP:
    for($i=0;$i<$ucs_str->length;$i++)
    {
        #printf "%x\n", $ucs_str->substr($i,1)->ord;
#        if ($ucs_str->substr($i,1)->ord == 0x20)
        if ($ucs_str->substr($i,1)->ord == 0x0)
        {
            return $kp_str;
        }
        
#        if (is_ascii($ucs_str->substr($i,1)->ord))
        if (!is_kannada($ucs_str->substr($i,1)->ord))
        {
#            $kp_str.=$ucs_str->substr($i,1)->utf8;
            $kp_str.=$ucs_str->substr($i,1)->latin1;
            next LOOP;
        }
        else
        {
            my $encountered;
            my $ucs_char=$ucs_str->substr($i,1)->ord;
            my $ucs_next_char=0;
            my $ucs_next_next_char=0;

            if ($i+1 < $ucs_str->length)
            {
                $ucs_next_char=$ucs_str->substr($i+1,1)->ord;
            }

            if ($i+2 < $ucs_str->length)
            {
                $ucs_next_next_char=$ucs_str->substr($i+2,1)->ord;
            }
            
            if(is_vowel($ucs_char))
            {
                $kp_str .= $kpvowels[$ucs_char - 0x0c85];
                if($ucs_char == 0x0c85)
                {
                    if($ucs_next_char==0x0c82) { $kp_str.=$kpanuvis[0]; $i++; }
                    if($ucs_next_char==0x0c83) { $kp_str.=$kpanuvis[1]; $i++; }
                }
                next LOOP;
            }
            elsif(is_consonant($ucs_char))
            {
                if(is_consonant($ucs_next_char) || ($ucs_next_char == 0x20) ||
                    ($ucs_next_char == 0x0) || (is_digit($ucs_next_char)))
                {
                    $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                    $kp_str.="";
                    next LOOP;
                }
                elsif(is_dependent($ucs_next_char))
                {
                    if($ucs_next_char == 0x0cbf)
                    {
                        $kp_str.=$kpIdeps[$ucs_char - 0x0c95];
                        $i++;
                        next LOOP;
                    }
                    elsif($ucs_next_char == 0x0cc0)
                    {
                        $kp_str.=$kpIdeps[$ucs_char - 0x0c95];
                        $kp_str.=$kpdepend[$ucs_next_char - 0x0cbe];
                        $i++;
                        next LOOP;
                    }
                    elsif(($ucs_next_char == 0x0cc1) && ($ucs_char == 0x0cb5))
                    {
                        $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                        $kp_str.="";
                        $i++;
                        next LOOP;
                    }
                    elsif(($ucs_next_char == 0x0cc2) && ($ucs_char == 0x0cb5))
                    {
                        $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                        $kp_str.="";
                        $i++;
                        next LOOP;
                    }
                    elsif(($ucs_next_char == 0x0c82) ||
                        ($ucs_next_char == 0x0c83))
                    {
                        $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                        $kp_str.="";
                        $kp_str.=$kpanuvis[$ucs_next_char - 0x0c82];
                        $i++;
                        next LOOP;
                    }
                    elsif($ucs_next_char == 0x0ccd)
                    {
                        if( ($ucs_next_next_char == 0x20) ||
                            ($ucs_next_next_char == 0x0))
                        {
                            $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                            $kp_str.=$kpdepend[$ucs_next_char - 0x0cbe];
                            $i++;
                            next LOOP;
                        }
                        elsif(is_consonant($ucs_next_next_char))
                        {
                            my $ucs_n3_char = 0;
                            
                            if($i+3 < $ucs_str->length)
                            {
                                $ucs_n3_char=$ucs_str->substr($i+3,1)->ord;
                            }

                            if(is_consonant($ucs_n3_char) || 
                                ($ucs_n3_char == 0x0c82) ||
                                ($ucs_n3_char == 0x0c83) ||
                                ($ucs_n3_char == 0x20) ||
                                ($ucs_n3_char == 0x0))
                            {
                                $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                                $kp_str.="";
                                $kp_str.=$kpvottus[$ucs_next_next_char - 0x0c95];
                                $i+=2;
                                next LOOP;
                            }
                            elsif(is_dependent($ucs_n3_char))
                            {
                                if($ucs_n3_char==0x0cbf)
                                {
                                    $kp_str.=$kpIdeps[$ucs_char - 0x0c95];
                                    $kp_str.=$kpvottus[$ucs_next_next_char - 0x0c95];
                                    $i+=3;
                                    next LOOP;
                                }
                                elsif($ucs_n3_char==0x0cc0)
                                {
                                    $kp_str.=$kpIdeps[$ucs_char - 0x0c95];
                                    $kp_str.=$kpvottus[$ucs_next_next_char - 0x0c95];
                                    $kp_str.=$kpdepend[$ucs_n3_char - 0x0cbe];
                                    $i+=3;
                                    next LOOP;
                                }
                                else
                                {
                                    $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                                    $kp_str.=$kpdepend[$ucs_n3_char - 0x0cbe];
                                    $kp_str.=$kpvottus[$ucs_next_next_char - 0x0c95];
                                    $i+=3;
                                    next LOOP;
                                }
                            }
                            else
                            {
                                return "";
                            }
                        }
                        else
                        {
                            return "";
                        }
                    }
                    else
                    {
                        $kp_str.=$kpconsonants[$ucs_char - 0x0c95];
                        $kp_str.=$kpdepend[$ucs_next_char - 0x0cbe];
                        $i++;
                        next LOOP;
                    }
                }
                else
                {
                    return "";
                }
#                elsif(($ucs_char == 0x0c82) || ($ucs_char == 0x0c83))
#                {
#                    $kp_str.=$kpanuvis[$ucs_char - 0x0c82];
#                    next LOOP;
#                }
            }
            elsif(is_digit($ucs_char))
            {
                $kp_str .= $kpdigits[$ucs_char - 0x0ce6];
                next LOOP;
            }
            elsif(($ucs_char==0x0c82 ) || ($ucs_char==0x0c83))
            {
                $kp_str.=$kpanuvis[$ucs_char - 0x0c82];
                next LOOP;
            }
            else
            {
                return "";
            }
        }
    }

    return $kp_str;
}
