package hItrans2utf;
use Unicode::String qw(utf8 latin1 ucs2 uchr);

# Basic vowels and consonants
%itranstab = (
    "a"     => 0x0905,
    "A"     => 0x0906,
    "i"     => 0x0907,
    "I"     => 0x0908,
    "u"     => 0x0909,
    "U"     => 0x090a,
    "RRi"   => 0x090b,
    "RRI"   => 0x0960,
    "e"     => 0x090f,
    "ai"    => 0x0910,
    "o"     => 0x0913,
    "au"    => 0x0914,
    ".n"    => 0x0902,
    "H"     => 0x0903,
    "0"     => 0x0966,
    "1"     => 0x0967,
    "2"     => 0x0968,
    "3"     => 0x0969,
    "4"     => 0x096a,
    "5"     => 0x096b,
    "6"     => 0x096c,
    "7"     => 0x096d,
    "8"     => 0x096e,
    "9"     => 0x096f,
    "k"    => 0x0915,
    "kh"   => 0x0916,
    "g"    => 0x0917,
    "gh"   => 0x0918,
    "~N"   => 0x0919,
    "ch"   => 0x091a,
    "Ch"   => 0x091b,
    "j"    => 0x091c,
    "jh"   => 0x091d,
    "~n"   => 0x091e,
    "T"    => 0x091f,
    "Th"   => 0x0920,
    "D"    => 0x0921,
    "Dh"   => 0x0922,
    "N"    => 0x0923,
    "t"    => 0x0924,
    "th"   => 0x0925,
    "d"    => 0x0926,
    "dh"   => 0x0927,
    "n"    => 0x0928,
    "p"    => 0x092a,
    "ph"   => 0x092b,
    "b"    => 0x092c,
    "bh"   => 0x092d,
    "m"    => 0x092e,
    "y"    => 0x092f,
    "r"    => 0x0930,
    "l"    => 0x0932,
    "L"    => 0x0933,
    "v"    => 0x0935,
    "sh"   => 0x0936,
    "Sh"   => 0x0937,
    "s"    => 0x0938,
    "h"    => 0x0939,
    "{}"    => 0x094d
);

# Dependent vowels
%itransdeptab = (
    "A"     => 0x093e,
    "i"     => 0x093f,
    "I"     => 0x0940,
    "u"     => 0x0941,
    "U"     => 0x0942,
    "RRi"   => 0x0943,
    "RRI"   => 0x0944,
    "e"     => 0x0947,
#    "E"     => 0x0947,
    "ai"    => 0x0948,
    "o"     => 0x094b,
#    "O"     => 0x0ccb,
    "au"    => 0x094c,
    "{}"    => 0x094d,
    ".n"    => 0x0902,
    "H"    => 0x0903
);

sub itrans2utf {
    my $itrans_str = shift;
    my $token_length = 1;
    my $token_begin = 0;
#    my $utf_str = "";
    my $utf_str = Unicode::String->new("");
    my $state = "independent";
    my $encountered = "";

LOOP:
    while ($token_begin < length ($itrans_str)) 
    {
        if($token_length > 3) { return ""; }

        if (substr($itrans_str,$token_begin,1) eq " ")
        {
#            $utf_str.=" ";
            $utf_str->append(uchr(0x20));
            $token_begin++;
        }

        if (substr($itrans_str,$token_begin,1) eq ",")
        {
            $utf_str->append(uchr(0x20)); # ' '  
            $utf_str->append(uchr(0x23)); # '#' 
            $utf_str->append(uchr(0x47)); # 'G'
            $utf_str->append(uchr(0x31)); # '1'
            $utf_str->append(uchr(0x35)); # '5'
            $utf_str->append(uchr(0x30)); # '0'
            $utf_str->append(uchr(0x30)); # '0'
            $utf_str->append(uchr(0x30)); # '0'
            $utf_str->append(uchr(0x23)); # '#'
            $utf_str->append(uchr(0x20)); # ' '
            $token_begin++;
        }

        if (exists $itranstab{substr($itrans_str,$token_begin,$token_length)})
        {
            my $char = substr($itrans_str,$token_begin,$token_length);

            if ($char=~/^[aAiIuUReoH\.\{]/) { $encountered = "vowel"; }
            else { $encountered = "consonant"; }

            ($char=~/^[kgcjtTdDpbsS]/) && ($token_length == 1) && do
            {
                if(substr($itrans_str,$token_begin+1,1) eq "h")
                {
                    $token_length++;
                }
            };

            if ($char eq "a")
            {
                if($token_begin+1 < length($itrans_str))
                {
                    if((substr($itrans_str,$token_begin+1,1) eq "i") or
                       (substr($itrans_str,$token_begin+1,1) eq "u"))
                    {
                        $token_length++;
                    }
                    elsif($state eq "dependent")
                    {
                        $token_length = 1;
                        $token_begin++;
                        $state = "independent";
                        next LOOP;
                    }
                }
                elsif($state eq "dependent")
                {
                    $token_length = 1;
                    $token_begin++;
                    $state = "independent";
                    next LOOP;
                }
            }

            my $uchar;
            
            if(($encountered eq "vowel") && ($state eq "dependent"))
            {
                $uchar=$itransdeptab{substr($itrans_str,$token_begin,$token_length)};
                $state = "independent";
            }
            else
            {
                $uchar=$itranstab{substr($itrans_str,$token_begin,$token_length)};
                if($encountered eq "vowel") { $state = "independent"; }
                else { $state = "dependent"; }
            }
            my $us = uchr($uchar);
#            $utf_str .= $us->utf8;
            $utf_str->append(uchr($uchar));
            $token_begin += $token_length;
            $token_length = 1;
        }
        else
        {
            $token_length++;
        }
    }

#    return $utf_str;
    return $utf_str->utf8;
}
