package kItrans2utf;
use Unicode::String qw(utf8 latin1 ucs2 uchr);

# Basic vowels and consonants
%itranstab = (
    "a"     => 0x0c85,
    "A"     => 0x0c86,
    "i"     => 0x0c87,
    "I"     => 0x0c88,
    "u"     => 0x0c89,
    "U"     => 0x0c8a,
    "RRi"   => 0x0c8b,
    "RRI"   => 0x0ce0,
    "e"     => 0x0c8e,
    "E"     => 0x0c8f,
    "ai"    => 0x0c90,
    "o"     => 0x0c92,
    "O"     => 0x0c93,
    "au"    => 0x0c94,
    ".n"    => 0x0c82,
    "H"     => 0x0c83,
    "0"     => 0x0ce6,
    "1"     => 0x0ce7,
    "2"     => 0x0ce8,
    "3"     => 0x0ce9,
    "4"     => 0x0cea,
    "5"     => 0x0ceb,
    "6"     => 0x0cec,
    "7"     => 0x0ced,
    "8"     => 0x0cee,
    "9"     => 0x0cef,
    "k"    => 0x0c95,
    "kh"   => 0x0c96,
    "g"    => 0x0c97,
    "gh"   => 0x0c98,
    "~N"   => 0x0c99,
    "ch"   => 0x0c9a,
    "Ch"   => 0x0c9b,
    "j"    => 0x0c9c,
    "jh"   => 0x0c9d,
    "~n"   => 0x0c9e,
    "T"    => 0x0c9f,
    "Th"   => 0x0ca0,
    "D"    => 0x0ca1,
    "Dh"   => 0x0ca2,
    "N"    => 0x0ca3,
    "t"    => 0x0ca4,
    "th"   => 0x0ca5,
    "d"    => 0x0ca6,
    "dh"   => 0x0ca7,
    "n"    => 0x0ca8,
    "p"    => 0x0caa,
    "ph"   => 0x0cab,
    "b"    => 0x0cac,
    "bh"   => 0x0cad,
    "m"    => 0x0cae,
    "y"    => 0x0caf,
    "r"    => 0x0cb0,
    "l"    => 0x0cb2,
    "L"    => 0x0cb3,
    "v"    => 0x0cb5,
    "sh"   => 0x0cb6,
    "Sh"   => 0x0cb7,
    "s"    => 0x0cb8,
    "h"    => 0x0cb9,
    "{}"    => 0x0ccd
);

# Dependent vowels
%itransdeptab = (
    "A"     => 0x0cbe,
    "i"     => 0x0cbf,
    "I"     => 0x0cc0,
    "u"     => 0x0cc1,
    "U"     => 0x0cc2,
    "RRi"   => 0x0cc3,
    "RRI"   => 0x0cc4,
    "e"     => 0x0cc6,
    "E"     => 0x0cc7,
    "ai"    => 0x0cc8,
    "o"     => 0x0cca,
    "O"     => 0x0ccb,
    "au"    => 0x0ccc,
    "{}"    => 0x0ccd,
    ".n"    => 0x0c82,
    "H"    => 0x0c83
);

sub itrans2utf {
    my $itrans_str = shift;
    my $token_length = 1;
    my $token_begin = 0;
#    my $utf_str = "";
    my $utf_str = Unicode::String->new("");
    my $state = "independent";
    my $encountered = "";

LOOP:
    while ($token_begin < length ($itrans_str)) 
    {
        if($token_length > 3) { return ""; }

        if (substr($itrans_str,$token_begin,1) eq " ")
        {
#            $utf_str.=" ";
            $utf_str->append(uchr(0x20));
            $token_begin++;
        }

        if (exists $itranstab{substr($itrans_str,$token_begin,$token_length)})
        {
            my $char = substr($itrans_str,$token_begin,$token_length);

            if ($char=~/^[aAiIuUReEoOH\.\{]/) { $encountered = "vowel"; }
            else { $encountered = "consonant"; }

            ($char=~/^[kgcjtTdDpbsS]/) && ($token_length == 1) && do
            {
                if(substr($itrans_str,$token_begin+1,1) eq "h")
                {
                    $token_length++;
                }
            };

            if ($char eq "a")
            {
                if($token_begin+1 < length($itrans_str))
                {
                    if((substr($itrans_str,$token_begin+1,1) eq "i") or
                       (substr($itrans_str,$token_begin+1,1) eq "u"))
                    {
                        $token_length++;
                    }
                    elsif($state eq "dependent")
                    {
                        $token_length = 1;
                        $token_begin++;
                        $state = "independent";
                        next LOOP;
                    }
                }
                elsif($state eq "dependent")
                {
                    $token_length = 1;
                    $token_begin++;
                    $state = "independent";
                    next LOOP;
                }
            }

            my $uchar;
            
            if(($encountered eq "vowel") && ($state eq "dependent"))
            {
                $uchar=$itransdeptab{substr($itrans_str,$token_begin,$token_length)};
                $state = "independent";
            }
            else
            {
                $uchar=$itranstab{substr($itrans_str,$token_begin,$token_length)};
                if($encountered eq "vowel") { $state = "independent"; }
                else { $state = "dependent"; }
            }
            my $us = uchr($uchar);
#            $utf_str .= $us->utf8;
            $utf_str->append(uchr($uchar));
            $token_begin += $token_length;
            $token_length = 1;
        }
        else
        {
            $token_length++;
        }
    }

#    return $utf_str;
    return $utf_str->utf8;
}
