package sTable;
use strict;
use base qw(Tk::Frame);
use vars qw($VERSION $DEBUG);
$VERSION = '1.0';
$DEBUG=0;

Construct Tk::Widget 'sTable';

sub ClassInit{shift}

sub Populate{
	  my ($t,$args) = @_;
	  $t->SUPER::Populate($args);
	  $t->ConfigSpecs(
                '-rows'                => [METHOD => 'rows','Rows',3],
                '-columns'             => [METHOD => 'columns','Columns',3],
                '-highlightthickness'  => [SELF => 'highlightThickness','HighlightThickness',0]
          );
	  $t->{'Width'}  = [];
	  $t->{'Height'} = [];
	  $t->{'Row'}    = [];
	  $t->{'child'}  = {};
}

sub put{
        my ($t,$row,$col,$w) = @_;
        $w = $t->Label(-text => $w, -anchor=>'w', -relief=>'sunken', -bd=>1) 
                                                               unless (ref $w);
        $t->ManageGeometry($w);
        unless (defined $t->{Row}->[$row]) {
                $t->{Row}->[$row] = [];
                $t->{Height}->[$row] = 0;
        }
        unless (defined $t->{Width}->[$col]) {
                $t->{Width}->[$col] = 0;
	}
        $t->{Row}->[$row][$col] = $w;
        $t->{child}{$w->PathName} = [$row,$col];
        _update_geometry($t, $w);
}

sub place{
	  my $t    = shift;
	  return unless Tk::Exists($t);
	  my $rows   = @{$t->{Row}};
          my $border = $t->cget(-highlightthickness);
          my $W = _sum_arr($t->{Width}) + 2*$border;
          my $H = _sum_arr($t->{Height})+ 2*$border;
          eval{$t->GeometryRequest($W,$H)};
          my $row=@{$t->{Height}};          
          my $col=@{$t->{Width}};
          my ($i, $j)=(0,0);
          my $y=$border;
          print "|| $H $W ||\n" if $DEBUG;
          for($i=0;$i<$row;$i++){
                  my $x=$border;
                  my $h = $t->{Height}->[$i];
                  for($j=0;$j<$col;$j++){
                          my $s = $t->{Row}->[$i][$j];
                          if(!defined $s){next}
                          $s->UnmapWindow;
                          my $w = $t->{Width}->[$j];
                          print "|| $i, $j => [$h $w]||\n" if $DEBUG;
                          $s->MoveResizeWindow($x,$y,$w,$h);
                          print ">> $x $y : $h $w <<\n" if $DEBUG;
                          if(ref($s)=~/sTable/){$s->propagate}
                          $s->MapWindow;
                          $x+=$w;
                  }
                  $y += $h;
          }          
}


sub propagate {
	  my $t   = shift;
          my $border = $t->cget(-highlightthickness);          
          my $rw = $t->ReqWidth;
	  my $rh = $t->ReqHeight;
          my $W  = $t->Width;
	  my $H  = $t->Height;
          print "\t<< $rh $rw :: $H $W>>\n" if $DEBUG;
          $W=$W-2*$border; $H=$H-2*$border; # true size 
          $rw=$rw-2*$border; $rh=$rh-2*$border; # true size
          _scale($t->{Width}, $W-$rw);
          _scale($t->{Height},$H-$rh);
          place($t);
}


sub SlaveGeometryRequest{}

sub _update_geometry{
	  my ($m,$s) = @_;
	  my ($row,$col) = @{$m->{child}->{$s->PathName}};
	  my $sw = $s->ReqWidth;
	  my $sh = $s->ReqHeight;
          print ";; $row, $col => $sh $sw\n" if $DEBUG;
	  if ($sw > $m->{Width}->[$col]) {$m->{Width}->[$col] = $sw;}
	  if ($sh > $m->{Height}->[$row]) {$m->{Height}->[$row] = $sh}
}

sub _sum_arr {
	  my $a = shift; my $sum = 0;
	  for (@$a) {$sum += $_}
	  return $sum;
}

sub _scale {
        my ($a, $diff)=@_;
        OUT:
        while(1){
                for(@$a){
                        if($diff==0){last OUT}
                        $_++;
                        $diff--;
                        
                }
        }
}


# debug 
sub debug {$DEBUG=1}

#


1;
# configure methods
sub rows {shift->_cget('-rows')}
sub columns {shift->_cget('-columns')}
